@@ -7,8 +7,6 @@ from filecmp import dircmp
 from pathlib import Path, PosixPath
 from typing import List, Optional, Set, Tuple
 
-import requests
-
 from . import get_version
 
 
@@ -35,17 +33,33 @@ class DjangoMigrationsDiff:
         self._comparison = None
         self._apps = None
         self._spacing = None
+        self._statistics = None
 
         self._number_only = False
+        self._max_width = 120
 
     def run(self):
         """Run application."""
 
-        # "Numbers only" keyword argument
+        # "Numbers only" flag
         if '--number' in self.args:
             self._number_only = True
             self.args.remove('--number')
 
+        # "Max width" flag
+        if '--width' in self.args:
+            try:
+                index = self.args.index('--width')
+                width_value = self.args[index + 1]
+                self._max_width = int(width_value)
+                self.args.pop(index)  # remove flag
+                self.args.pop(index)  # remove flag value
+            except Exception:
+                exit(self.print('\n<r>Invalid --width flag value.\nExample: --width 128</r>'))
+
+            if self._max_width < 30:
+                exit(self.print('\n<r>Width should be at least 30 to be readable'))
+
         # Show help
         if not self.args or 'help' in self.args[0]:
             return self.show_help()
@@ -71,6 +85,7 @@ class DjangoMigrationsDiff:
 
     def create_snapshot(self, name: str):
         """Create new snapshot (or update old one)."""
+        name = self.escape_characters(name)
         new_snapshot_dir, created = None, None
         total_apps, total_files = 0, 0
 
@@ -96,15 +111,13 @@ class DjangoMigrationsDiff:
                 total_apps += 1
 
         if not total_files:
-            return self.print(
-                f'\n<r>Can\'t find any migration in {self.current_dir}</r>'
-            )
+            return self.print(f'\n<r>Can\'t find any migration in {self.current_dir}</r>')
 
         action = 'created' if created else 'updated'
         return self.print(
             f'\nSnapshot <g>{name}</g> successfully {action}:\n'
             f'  - {total_apps} applications\n'
-            f'  - {total_files} migrations'
+            f'  - {total_files} migrations',
         )
 
     def remove_snapshots(self, *names: Tuple[str, ...]):
@@ -113,7 +126,7 @@ class DjangoMigrationsDiff:
             return self.print(
                 '\nmdiff rm <g>all</g>\t\t-- Remove all snapshots'
                 '\nmdiff rm <g><snapshot></g>\t-- Remove specific snapshots '
-                '(separate names by space)'
+                '(separate names by space)',
             )
 
         # Remove all snapshots
@@ -121,45 +134,42 @@ class DjangoMigrationsDiff:
             if not self.show_all_snapshots():
                 return
 
-            self.input(
-                f'\nRemove all snapshots? Press Enter to continue...', end=''
-            )
+            self.input('\nRemove all snapshots? Press Enter to continue...', end='')
             shutil.rmtree(self.snapshots_dir)
             return self.print('<g>OK!</g>')
 
         # Remove specific snapshots
         self.print()
+        names = tuple(self.escape_characters(n) for n in names)
         for name in names:
             self.print(f'Snapshot {name} - ', end='')
             snapshot_dir = self.snapshots_dir / name
             if snapshot_dir.exists():
                 shutil.rmtree(snapshot_dir)
-                self.print(f'<g>deleted</g>')
+                self.print('<g>deleted</g>')
             else:
-                self.print(f'<r>not found</r>')
+                self.print('<r>not found</r>')
 
     def compare_snapshots(self, *names: Tuple[str, ...]):
         """Compare migrations between several snapshots."""
-        self.names = names
+        self.names = tuple(self.escape_characters(n) for n in names)
         if len(self.names) > 2:
             # NOTE: maybe it's not needed to anyone?
-            return self.print(
-                '\n<r>Sorry, only 2 snapshots can be compared yet</r>'
-            )
+            return self.print('\n<r>Sorry, only 2 snapshots can be compared yet</r>')
 
         if self._number_only:
             return print(len(self.comparison))
 
         if not self.comparison:
             return self.print(
-                f'\nSnapshots <g>{names[0]}</g> and <g>{names[1]}</g> '
-                'are equal!'
+                f'\nSnapshots <g>{self.names[0]}</g> and '
+                f'<g>{self.names[1]}</g> are equal!',
             )
 
         self.print_line(left='┌', delimiter='┬', right='┐')
         self.print_line(
             self.app_title,
-            *[n.upper() for n in names],
+            *[n.upper() for n in self.names],
             wraps=['B'] * 3,
         )
 
@@ -167,7 +177,7 @@ class DjangoMigrationsDiff:
             self.print_line(left='├', delimiter='┼', right='┤')
             for index, filenames in enumerate(sorted(
                 files,
-                key=lambda x: x[0] if x[0] != self.empty else x[1]
+                key=lambda x: x[0] if x[0] != self.empty else x[1],
             )):
                 wraps = [None]  # app name is not wrapped
                 if filenames[0] == filenames[1]:
@@ -178,6 +188,11 @@ class DjangoMigrationsDiff:
                 self.print_line('' if index else app, *filenames, wraps=wraps)
 
         self.print_line(left='└', delimiter='┴', right='┘')
+        self.print(
+            f' Stats for snapshot {self.names[1]}: <g>+{self._statistics[0]}</g> '
+            f'<r>-{self._statistics[1]}</r> '
+            f'<y>*{self._statistics[2]}</y>',
+        )
 
     def show_all_snapshots(self) -> bool:
         """Show all existing snapshots.
@@ -191,9 +206,7 @@ class DjangoMigrationsDiff:
             return False
 
         spacing = max(len(max(snapshots, key=len)), 4)
-        self.print(
-            f'\n<g>NAME{" " * spacing}APPS\tFILES\tDATE\t\t   SIZE</g>\n'
-        )
+        self.print(f'\n<g>NAME{" " * spacing}APPS\tFILES\tDATE\t\t   SIZE</g>\n')
         for snapshot in snapshots:
             total_apps, total_files, total_size = 0, 0, 0
             snapshot_dir = self.snapshots_dir / snapshot
@@ -205,13 +218,13 @@ class DjangoMigrationsDiff:
                     total_size += sum(
                         (root / filename).stat().st_size
                         for filename in files
-                    ) / 1000
+                    )
 
             self.print(
                 f'{snapshot}{" " * (spacing - len(snapshot) + 4)}'
                 f'{total_apps}\t{total_files}\t'
                 f'{self.get_created_date(snapshot_dir)}   '
-                f'{int(total_size)} kB'
+                f'{self.format_total_size(total_size)}',
             )
 
         return True
@@ -231,7 +244,7 @@ class DjangoMigrationsDiff:
         if not actual_version:
             self.print('\n<r>Can\'t get latest version from server</r>')
         if actual_version == self.version:
-            self.print('\nThis version is up-to-date')
+            self.print('\nThis version is <g>up-to-date</g>')
 
         self.set_last_check()
 
@@ -250,10 +263,13 @@ class DjangoMigrationsDiff:
         actual_version = None
 
         try:
-            response = requests.get(
-                f'https://pypi.org/pypi/{self.package_name}/json/',
-                timeout=10,
-            )
+            import requests
+        except (ModuleNotFoundError, ImportError):
+            self.print('\n<r>Package "requests" is not found. Cannot get actual version.</r>')
+            exit()
+
+        try:
+            response = requests.get(f'https://pypi.org/pypi/{self.package_name}/json/', timeout=10)
             if response and response.status_code == 200:
                 data = response.json()
                 actual_version = data['info']['version']
@@ -268,9 +284,7 @@ class DjangoMigrationsDiff:
             self._spacing = [len(msg) + 1]
             self.print_line(left='\n┌', delimiter='┬', right='┐')
             self.print_line(msg, wraps=['y'])
-            self.print_line(
-                f'pip3 install --upgrade {self.package_name}', wraps=['y']
-            )
+            self.print_line(f'pip3 install --upgrade {self.package_name}', wraps=['y'])
             self.print_line(left='└', delimiter='┴', right='┘')
 
         return actual_version
@@ -296,6 +310,10 @@ class DjangoMigrationsDiff:
 
     # Helpers
 
+    def escape_characters(self, value: str) -> str:
+        """Remove restricted characters from value."""
+        return value.replace('/', '-')
+
     @property
     def current_dir(self) -> PosixPath:
         """Directory with Django project to inspect."""
@@ -371,8 +389,23 @@ class DjangoMigrationsDiff:
                     for filename in compare.diff_files:
                         self._comparison[app].append((filename, filename))
 
+            self.update_statistics()
         return self._comparison
 
+    def update_statistics(self):
+        """Update `self._statistics` values."""
+        # Added, deleted, updated
+        self._statistics = [0, 0, 0]
+
+        for app in self._comparison.values():
+            for line in app:
+                for i, value in enumerate(line):
+                    if value == self.empty:
+                        self._statistics[i] += 1  # New migration or deleted migration
+
+                if len(set(line)) == 1:
+                    self._statistics[2] += 1  # Both changed
+
     @property
     def apps(self) -> Set[str]:
         """Get set of all available apps from all snapshots."""
@@ -413,7 +446,12 @@ class DjangoMigrationsDiff:
                     if name_2 > max_name_2:
                         max_name_2 = name_2
 
-            self._spacing = max_app + 1, max_name_1 + 1, max_name_2 + 1
+            self._spacing = [max_app + 1, max_name_1 + 1, max_name_2 + 1]
+
+            # Reduce spacing to fit max width limitation
+            while sum(self._spacing) > self._max_width:
+                max_index = self._spacing.index(max(self._spacing))
+                self._spacing[max_index] -= 1
 
         return self._spacing
 
@@ -425,11 +463,6 @@ class DjangoMigrationsDiff:
         """Create directory for new snapshot (and remove old one if exists.)"""
         new_snapshot_dir = self.snapshots_dir / name
         if new_snapshot_dir.exists():
-            date = self.get_created_date(new_snapshot_dir)
-            self.input(
-                f'\nSnapshot <g>{name}</g> ({date}) will be updated. '
-                'Press Enter to continue...', end=''
-            )
             shutil.rmtree(new_snapshot_dir)
             created = False
 
@@ -493,6 +526,9 @@ class DjangoMigrationsDiff:
             end = f'<D>{delimiter}</D>' if i + 1 != len(self.spacing) else ''
             if values:
                 value = values[i]
+                max_width = self.spacing[i]
+                if len(value) > max_width and value != self.empty:
+                    value = self.truncate_message(value, max_width, delimiter='...')
                 after_value = spacing - len(value)
                 if wraps:
                     wrapper = wraps[i]
@@ -521,6 +557,42 @@ class DjangoMigrationsDiff:
         date = datetime.fromtimestamp(path.stat().st_ctime)
         return date.strftime(self.time_format)
 
+    def format_total_size(self, size: int) -> str:
+        """Format total snapshot size."""
+        return f'{int(size / 1000)} Kb' if size > 1000 else f'{size} bytes'
+
+    def truncate_message(
+        self,
+        message: str,
+        max_length: int = 1000,
+        delimiter: str = ' ... ',
+        cut_end: bool = False,
+    ):
+        """Truncate long messages.
+
+        Examples of use:
+            >>> self.truncate_message('a' * 100, max_length=15)
+            'aaaaaaa ... aa'
+
+            >>> self.truncate_message('a' * 100, max_length=15, cut_end=True)
+            'aaaaaaaaaa ... '
+
+            >>> self.truncate_message('a' * 15, max_length=15)
+            'aaaaaaaaaaaaaaa'
+        """
+        if len(message) <= max_length:
+            return message
+
+        if cut_end:
+            return message[:max_length - len(delimiter)] + delimiter
+
+        first_part = max_length // 2
+        second_part = first_part - len(delimiter)
+        if second_part < 1:
+            return message[:first_part] + delimiter
+
+        return message[:first_part] + delimiter + message[-second_part:]
+
 
 def main():
     """Main function that runs application."""
